@@ -16,6 +16,8 @@
 
 package org.gradle.api.internal.changedetection.state;
 
+import org.gradle.api.file.FileTreeElement;
+
 import java.io.File;
 
 public interface FileSnapshotter {
@@ -24,4 +26,7 @@ public interface FileSnapshotter {
      */
     FileSnapshot snapshot(File file);
 
+
+    FileSnapshot snapshot(FileTreeElement fileDetails);
+
 }
