@@ -24,9 +24,7 @@ import org.gradle.api.internal.DefaultClassPathRegistry;
 import org.gradle.api.internal.classpath.ModuleRegistry;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.cache.CacheRepository;
-import org.gradle.cache.internal.CacheFactory;
-import org.gradle.cache.internal.DefaultCacheRepository;
-import org.gradle.cache.internal.DefaultCacheScopeMapping;
+import org.gradle.cache.internal.CacheRepositoryServices;
 import org.gradle.deployment.internal.DefaultDeploymentRegistry;
 import org.gradle.deployment.internal.DeploymentRegistry;
 import org.gradle.internal.Factory;
@@ -40,7 +38,6 @@ import org.gradle.plugin.use.internal.InjectedPluginClasspath;
 import org.gradle.process.internal.DefaultWorkerProcessFactory;
 import org.gradle.process.internal.WorkerProcessBuilder;
 import org.gradle.process.internal.child.WorkerProcessClassPathProvider;
-import org.gradle.util.GradleVersion;
 
 /**
  * Contains the services for a single build session, which could be a single build or multiple builds when in continuous mode.
@@ -59,6 +56,7 @@ public class BuildSessionScopeServices extends DefaultServiceRegistry {
             }
         });
         add(InjectedPluginClasspath.class, new InjectedPluginClasspath(injectedPluginClassPath));
+        addProvider(new CacheRepositoryServices(startParameter.getGradleUserHomeDir(), startParameter.getProjectCacheDir()));
     }
 
     DeploymentRegistry createDeploymentRegistry() {
@@ -76,15 +74,6 @@ public class BuildSessionScopeServices extends DefaultServiceRegistry {
             startParameter.getGradleUserHomeDir());
     }
 
-    protected CacheRepository createCacheRepository() {
-        CacheFactory factory = get(CacheFactory.class);
-        StartParameter startParameter = get(StartParameter.class);
-        DefaultCacheScopeMapping scopeMapping = new DefaultCacheScopeMapping(startParameter.getGradleUserHomeDir(), startParameter.getProjectCacheDir(), GradleVersion.current());
-        return new DefaultCacheRepository(
-            scopeMapping,
-            factory);
-    }
-
     protected ClassPathRegistry createClassPathRegistry() {
         return new DefaultClassPathRegistry(
             new DefaultClassPathProvider(get(ModuleRegistry.class)),
