@@ -22,6 +22,7 @@ import org.gradle.api.Task;
 import org.gradle.plugins.ide.eclipse.EclipsePlugin;
 import org.gradle.plugins.ide.eclipse.model.*;
 import org.gradle.plugins.ide.internal.tooling.eclipse.*;
+import org.gradle.plugins.ide.internal.tooling.java.DefaultInstalledJdk;
 import org.gradle.tooling.internal.gradle.DefaultGradleProject;
 import org.gradle.tooling.provider.model.ToolingModelBuilder;
 import org.gradle.util.GUtil;
@@ -136,6 +137,14 @@ public class EclipseModelBuilder implements ToolingModelBuilder {
             buildCommands.add(new DefaultEclipseBuildCommand(b.getName(), b.getArguments()));
         }
         eclipseProject.setBuildCommands(buildCommands);
+        EclipseJdt jdt = eclipseModel.getJdt();
+        if (jdt != null) {
+            eclipseProject.setJavaSourceSettings(new DefaultEclipseJavaSourceSettings()
+                .setSourceLanguageLevel(jdt.getSourceCompatibility())
+                .setTargetBytecodeVersion(jdt.getTargetCompatibility())
+                .setJdk(DefaultInstalledJdk.current())
+            );
+        }
 
         for (Project childProject : project.getChildProjects().values()) {
             populate(childProject);
