@@ -16,22 +16,15 @@
 
 package org.gradle.model.internal.manage.schema.extract;
 
-import org.gradle.model.internal.manage.schema.ModelValueSchema;
+import org.gradle.model.internal.manage.schema.ScalarValueSchema;
 import org.gradle.model.internal.type.ModelType;
 
 public class JdkValueTypeStrategy implements ModelSchemaExtractionStrategy {
 
     public <R> void extract(ModelSchemaExtractionContext<R> extractionContext) {
         ModelType<R> type = extractionContext.getType();
-        if (ScalarTypes.TYPES.contains(type)) {
-            extractionContext.found(new ModelValueSchema<R>(type));
-        } else {
-            for (ModelType<?> nonFinalType : ScalarTypes.NON_FINAL_TYPES) {
-                if (nonFinalType.isAssignableFrom(type)) {
-                    throw new InvalidManagedModelElementTypeException(extractionContext, "subclasses of " + nonFinalType + " are not supported");
-                }
-            }
+        if (ScalarTypes.isScalarType(type)) {
+            extractionContext.found(new ScalarValueSchema<R>(type));
         }
     }
-
 }
