@@ -15,13 +15,14 @@
  */
 package org.gradle.api.internal.resolve;
 
+import com.google.common.base.Objects;
 import com.google.common.base.Predicate;
+import com.google.common.collect.Lists;
 import org.gradle.api.UnknownProjectException;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.artifacts.component.LibraryBinaryIdentifier;
 import org.gradle.api.artifacts.component.LibraryComponentSelector;
 import org.gradle.api.internal.component.ArtifactType;
-import org.gradle.api.internal.tasks.DefaultTaskDependency;
 import org.gradle.internal.component.local.model.LocalComponentMetaData;
 import org.gradle.internal.component.local.model.PublishArtifactLocalArtifactMetaData;
 import org.gradle.internal.component.model.*;
@@ -38,13 +39,12 @@ import org.gradle.language.base.internal.model.VariantAxisCompatibilityFactory;
 import org.gradle.language.base.internal.model.VariantsMetaData;
 import org.gradle.language.base.internal.resolve.LibraryResolveException;
 import org.gradle.model.ModelMap;
-import org.gradle.model.internal.core.ModelPath;
 import org.gradle.model.internal.manage.schema.ModelSchemaStore;
 import org.gradle.model.internal.registry.ModelRegistry;
 import org.gradle.model.internal.type.ModelType;
 import org.gradle.platform.base.BinarySpec;
-import org.gradle.platform.base.ComponentSpecContainer;
-import org.gradle.platform.base.LibrarySpec;
+import org.gradle.platform.base.ComponentSpec;
+import org.gradle.platform.base.internal.BinarySpecInternal;
 
 import java.util.Collection;
 import java.util.Collections;
@@ -58,25 +58,25 @@ public class LocalLibraryDependencyResolver<T extends BinarySpec> implements Dep
     private final LibraryResolutionErrorMessageBuilder errorMessageBuilder;
     private final LocalLibraryMetaDataAdapter libraryMetaDataAdapter;
     private final Class<? extends BinarySpec> binaryType;
-    private final Predicate<LibrarySpec> binarySpecPredicate;
+    private final Predicate<ComponentSpec> binarySpecPredicate;
 
     public LocalLibraryDependencyResolver(
-            Class<T> binarySpecType,
-            ProjectModelResolver projectModelResolver,
-            List<VariantAxisCompatibilityFactory> selectorFactories,
-            VariantsMetaData variantsMetaData,
-            ModelSchemaStore schemaStore,
-            LocalLibraryMetaDataAdapter libraryMetaDataAdapter,
-            LibraryResolutionErrorMessageBuilder errorMessageBuilder) {
+        Class<T> binarySpecType,
+        ProjectModelResolver projectModelResolver,
+        List<VariantAxisCompatibilityFactory> selectorFactories,
+        VariantsMetaData variantsMetaData,
+        ModelSchemaStore schemaStore,
+        LocalLibraryMetaDataAdapter libraryMetaDataAdapter,
+        LibraryResolutionErrorMessageBuilder errorMessageBuilder) {
         this.projectModelResolver = projectModelResolver;
         this.libraryMetaDataAdapter = libraryMetaDataAdapter;
         this.matcher = new VariantsMatcher(selectorFactories, binarySpecType, schemaStore);
         this.errorMessageBuilder = errorMessageBuilder;
         this.variantsMetaData = variantsMetaData;
         this.binaryType = binarySpecType;
-        this.binarySpecPredicate = new Predicate<LibrarySpec>() {
+        this.binarySpecPredicate = new Predicate<ComponentSpec>() {
             @Override
-            public boolean apply(LibrarySpec input) {
+            public boolean apply(ComponentSpec input) {
                 return !input.getBinaries().withType(binaryType).isEmpty();
             }
         };
@@ -86,52 +86,93 @@ public class LocalLibraryDependencyResolver<T extends BinarySpec> implements Dep
     public void resolve(DependencyMetaData dependency, final BuildableComponentIdResolveResult result) {
         if (dependency.getSelector() instanceof LibraryComponentSelector) {
             LibraryComponentSelector selector = (LibraryComponentSelector) dependency.getSelector();
-            final String selectorProjectPath = selector.getProjectPath();
-            final String libraryName = selector.getLibraryName();
-            LibraryResolutionErrorMessageBuilder.LibraryResolutionResult resolutionResult = doResolve(selectorProjectPath, libraryName);
-            LibrarySpec selectedLibrary = resolutionResult.getSelectedLibrary();
-            if (selectedLibrary != null) {
-                Collection<BinarySpec> allBinaries = selectedLibrary.getBinaries().values();
-                Collection<? extends BinarySpec> compatibleBinaries = matcher.filterBinaries(variantsMetaData, allBinaries);
-                if (!allBinaries.isEmpty() && compatibleBinaries.isEmpty()) {
-                    // no compatible variant found
-                    result.failed(new ModuleVersionResolveException(selector, errorMessageBuilder.noCompatibleVariantErrorMessage(libraryName, allBinaries)));
-                } else if (compatibleBinaries.size() > 1) {
-                    result.failed(new ModuleVersionResolveException(selector, errorMessageBuilder.multipleCompatibleVariantsErrorMessage(libraryName, compatibleBinaries)));
-                } else {
-                    BinarySpec selectedBinary = compatibleBinaries.iterator().next();
-                    DefaultTaskDependency buildDependencies = new DefaultTaskDependency();
-                    buildDependencies.add(selectedBinary);
-                    LocalComponentMetaData metaData = libraryMetaDataAdapter.createLocalComponentMetaData(selectedBinary, buildDependencies, selectorProjectPath);
-                    result.resolved(metaData);
-                }
+            resolveLibraryAndChooseBinary(result, selector);
+        }
+    }
+
+    private void resolveLibraryAndChooseBinary(BuildableComponentIdResolveResult result, LibraryComponentSelector selector) {
+        final String selectorProjectPath = selector.getProjectPath();
+        final String libraryName = selector.getLibraryName();
+        final String variant = selector.getVariant();
+        LibraryResolutionErrorMessageBuilder.LibraryResolutionResult resolutionResult = doResolve(selectorProjectPath, libraryName);
+        ComponentSpec selectedLibrary = resolutionResult.getSelectedLibrary();
+        if (selectedLibrary != null) {
+            if (variant == null) {
+                selectMatchingVariant(result, selectedLibrary, selector, selectorProjectPath, libraryName);
+            } else {
+                selectExplicitlyProvidedVariant(result, selectedLibrary, selectorProjectPath, variant);
             }
-            if (!result.hasResult()) {
-                String message = resolutionResult.toResolutionErrorMessage(binaryType, selector);
-                ModuleVersionResolveException failure = new ModuleVersionResolveException(selector, new LibraryResolveException(message));
-                result.failed(failure);
+        }
+        if (!result.hasResult()) {
+            String message = resolutionResult.toResolutionErrorMessage(binaryType, selector);
+            ModuleVersionResolveException failure = new ModuleVersionResolveException(selector, new LibraryResolveException(message));
+            result.failed(failure);
+        }
+    }
+
+    private void selectExplicitlyProvidedVariant(BuildableComponentIdResolveResult result, ComponentSpec selectedLibrary, String selectorProjectPath, String variant) {
+        Collection<BinarySpec> allBinaries = selectedLibrary.getBinaries().values();
+        for (BinarySpec binarySpec : allBinaries) {
+            BinarySpecInternal binary = (BinarySpecInternal) binarySpec;
+            LibraryBinaryIdentifier id = binary.getId();
+            if (Objects.equal(variant, id.getVariant())) {
+                // TODO:Cedric This is not quite right. We assume that if we are asking for a specific binary, then we resolve to the assembly instead
+                // of the jar, but it should be somehow parametrized
+                LocalComponentMetaData metaData = libraryMetaDataAdapter.createLocalComponentMetaData(binary, selectorProjectPath, true);
+                result.resolved(metaData);
             }
         }
     }
 
+    private void selectMatchingVariant(BuildableComponentIdResolveResult result, ComponentSpec selectedLibrary, LibraryComponentSelector selector, String selectorProjectPath, String libraryName) {
+        Collection<BinarySpec> allBinaries = selectedLibrary.getBinaries().values();
+        Collection<? extends BinarySpec> compatibleBinaries = matcher.filterBinaries(variantsMetaData, allBinaries);
+        if (!allBinaries.isEmpty() && compatibleBinaries.isEmpty()) {
+            // no compatible variant found
+            result.failed(new ModuleVersionResolveException(selector, errorMessageBuilder.noCompatibleVariantErrorMessage(libraryName, allBinaries)));
+        } else if (compatibleBinaries.size() > 1) {
+            result.failed(new ModuleVersionResolveException(selector, errorMessageBuilder.multipleCompatibleVariantsErrorMessage(libraryName, compatibleBinaries)));
+        } else {
+            BinarySpec selectedBinary = compatibleBinaries.iterator().next();
+            LocalComponentMetaData metaData = libraryMetaDataAdapter.createLocalComponentMetaData(selectedBinary, selectorProjectPath, false);
+            result.resolved(metaData);
+        }
+    }
+
     private LibraryResolutionErrorMessageBuilder.LibraryResolutionResult doResolve(String projectPath,
                                                                                    String libraryName) {
         try {
             ModelRegistry projectModel = projectModelResolver.resolveProjectModel(projectPath);
-            ComponentSpecContainer components = projectModel.find(
-                ModelPath.path("components"),
-                ModelType.of(ComponentSpecContainer.class));
-            if (components != null) {
-                ModelMap<? extends LibrarySpec> libraries = components.withType(LibrarySpec.class);
-                return LibraryResolutionErrorMessageBuilder.LibraryResolutionResult.of(libraries.values(), libraryName, binarySpecPredicate);
-            } else {
-                return LibraryResolutionErrorMessageBuilder.LibraryResolutionResult.emptyResolutionResult();
+            LibraryResolutionErrorMessageBuilder.LibraryResolutionResult libraries = findLocalComponent(libraryName, projectModel);
+            if (libraries != null) {
+                return libraries;
             }
+
+            return LibraryResolutionErrorMessageBuilder.LibraryResolutionResult.emptyResolutionResult();
         } catch (UnknownProjectException e) {
             return LibraryResolutionErrorMessageBuilder.LibraryResolutionResult.projectNotFound();
         }
     }
 
+    private LibraryResolutionErrorMessageBuilder.LibraryResolutionResult findLocalComponent(String componentName, ModelRegistry projectModel) {
+        List<ComponentSpec> librarySpecs = Lists.newArrayList();
+        collectLocalComponents(projectModel, "components", librarySpecs);
+        collectLocalComponents(projectModel, "testSuites", librarySpecs);
+        if (librarySpecs.isEmpty()) {
+            return null;
+        }
+        return LibraryResolutionErrorMessageBuilder.LibraryResolutionResult.of(librarySpecs, componentName, binarySpecPredicate);
+    }
+
+    private void collectLocalComponents(ModelRegistry projectModel, String container, List<ComponentSpec> librarySpecs) {
+        ModelMap<ComponentSpec> components = projectModel.find(container, new ModelType<ModelMap<ComponentSpec>>() {
+        });
+        if (components != null) {
+            ModelMap<? extends ComponentSpec> libraries = components.withType(ComponentSpec.class);
+            librarySpecs.addAll(libraries.values());
+        }
+    }
+
     @Override
     public void resolve(ComponentIdentifier identifier, ComponentOverrideMetadata componentOverrideMetadata, BuildableComponentResolveResult result) {
         if (isLibrary(identifier)) {
