@@ -15,7 +15,6 @@
  */
 package org.gradle.nativeplatform.plugins;
 
-import com.google.common.collect.Lists;
 import org.apache.commons.lang.StringUtils;
 import org.gradle.api.*;
 import org.gradle.api.artifacts.repositories.ArtifactRepository;
@@ -33,13 +32,11 @@ import org.gradle.language.base.internal.LanguageSourceSetInternal;
 import org.gradle.language.base.internal.SourceTransformTaskConfig;
 import org.gradle.language.base.internal.registry.LanguageTransformContainer;
 import org.gradle.language.base.plugins.ComponentModelBasePlugin;
-import org.gradle.language.base.plugins.LifecycleBasePlugin;
 import org.gradle.language.nativeplatform.DependentSourceSet;
 import org.gradle.language.nativeplatform.HeaderExportingSourceSet;
 import org.gradle.language.nativeplatform.internal.DependentSourceSetInternal;
 import org.gradle.model.*;
 import org.gradle.model.internal.registry.ModelRegistry;
-import org.gradle.model.internal.type.ModelType;
 import org.gradle.nativeplatform.*;
 import org.gradle.nativeplatform.internal.*;
 import org.gradle.nativeplatform.internal.configure.NativeComponentRules;
@@ -49,19 +46,20 @@ import org.gradle.nativeplatform.internal.prebuilt.PrebuiltLibraryInitializer;
 import org.gradle.nativeplatform.platform.NativePlatform;
 import org.gradle.nativeplatform.platform.internal.DefaultNativePlatform;
 import org.gradle.nativeplatform.platform.internal.NativePlatforms;
-import org.gradle.nativeplatform.tasks.*;
-import org.gradle.nativeplatform.test.internal.NativeTestSuiteBinarySpecInternal;
+import org.gradle.nativeplatform.tasks.CreateStaticLibrary;
+import org.gradle.nativeplatform.tasks.LinkSharedLibrary;
+import org.gradle.nativeplatform.tasks.PrefixHeaderFileGenerateTask;
 import org.gradle.nativeplatform.toolchain.internal.DefaultNativeToolChainRegistry;
 import org.gradle.nativeplatform.toolchain.internal.NativeToolChainRegistryInternal;
 import org.gradle.nativeplatform.toolchain.internal.PreCompiledHeader;
 import org.gradle.platform.base.*;
-import org.gradle.platform.base.internal.BinaryNamingScheme;
 import org.gradle.platform.base.internal.PlatformResolvers;
 
 import javax.inject.Inject;
 import java.io.File;
-import java.util.List;
-import java.util.concurrent.Callable;
+
+import static org.gradle.model.internal.core.ModelNodes.withType;
+import static org.gradle.model.internal.core.NodePredicate.allDescendants;
 
 /**
  * A plugin that sets up the infrastructure for defining native binaries.
@@ -84,7 +82,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         project.getExtensions().create("flavors", DefaultFlavorContainer.class, instantiator);
         project.getExtensions().create("toolChains", DefaultNativeToolChainRegistry.class, instantiator);
 
-        modelRegistry.getRoot().applyToAllLinksTransitive(ModelType.of(NativeComponentSpec.class), NativeComponentRules.class);
+        modelRegistry.getRoot().applyTo(allDescendants(withType(NativeComponentSpec.class)), NativeComponentRules.class);
     }
 
     static class Rules extends RuleSource {
@@ -98,6 +96,11 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             builder.defaultImplementation(DefaultNativeLibrarySpec.class);
         }
 
+        @ComponentType
+        void registerTargetedNativeComponent(ComponentTypeBuilder<TargetedNativeComponent> builder) {
+            builder.internalView(TargetedNativeComponentInternal.class);
+        }
+
         @Model
         Repositories repositories(ServiceRegistry serviceRegistry, FlavorContainer flavors, PlatformContainer platforms, BuildTypeContainer buildTypes) {
             Instantiator instantiator = serviceRegistry.get(Instantiator.class);
@@ -145,16 +148,19 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         @BinaryType
         void registerSharedLibraryBinaryType(BinaryTypeBuilder<SharedLibraryBinarySpec> builder) {
             builder.defaultImplementation(DefaultSharedLibraryBinarySpec.class);
+            builder.internalView(SharedLibraryBinarySpecInternal.class);
         }
 
         @BinaryType
         void registerStaticLibraryBinaryType(BinaryTypeBuilder<StaticLibraryBinarySpec> builder) {
             builder.defaultImplementation(DefaultStaticLibraryBinarySpec.class);
+            builder.internalView(StaticLibraryBinarySpecInternal.class);
         }
 
         @BinaryType
         void registerNativeExecutableBinaryType(BinaryTypeBuilder<NativeExecutableBinarySpec> builder) {
             builder.defaultImplementation(DefaultNativeExecutableBinarySpec.class);
+            builder.internalView(NativeExecutableBinarySpecInternal.class);
         }
 
         @Finalize
@@ -183,11 +189,10 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             componentSpecs.afterEach(new Action<NativeComponentSpec>() {
                 @Override
                 public void execute(NativeComponentSpec componentSpec) {
-                    componentSpec.getSources().withType(LanguageSourceSet.class).afterEach(new Action<LanguageSourceSet>() {
+                    componentSpec.getSources().withType(LanguageSourceSetInternal.class).afterEach(new Action<LanguageSourceSetInternal>() {
                         @Override
-                        public void execute(LanguageSourceSet languageSourceSet) {
-                            LanguageSourceSetInternal internalSourceSet = (LanguageSourceSetInternal) languageSourceSet;
-                            Task generatorTask = internalSourceSet.getGeneratorTask();
+                        public void execute(LanguageSourceSetInternal languageSourceSet) {
+                            Task generatorTask = languageSourceSet.getGeneratorTask();
                             if (generatorTask != null) {
                                 languageSourceSet.builtBy(generatorTask);
                                 maybeSetSourceDir(languageSourceSet.getSource(), generatorTask, "sourceDir");
@@ -206,15 +211,14 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
             componentSpecs.afterEach(new Action<NativeComponentSpec>() {
                 @Override
                 public void execute(final NativeComponentSpec componentSpec) {
-                    componentSpec.getSources().withType(DependentSourceSet.class).afterEach(new Action<DependentSourceSet>() {
+                    componentSpec.getSources().withType(DependentSourceSetInternal.class).afterEach(new Action<DependentSourceSetInternal>() {
                         @Override
-                        public void execute(DependentSourceSet dependentSourceSet) {
+                        public void execute(DependentSourceSetInternal dependentSourceSet) {
                             if (dependentSourceSet.getPreCompiledHeader() != null) {
-                                DependentSourceSetInternal internalSourceSet = (DependentSourceSetInternal) dependentSourceSet;
                                 String prefixHeaderDirName = String.format("tmp/%s/%s/prefixHeaders", componentSpec.getName(), dependentSourceSet.getName());
                                 File prefixHeaderDir = new File(buildDir, prefixHeaderDirName);
                                 File prefixHeaderFile = new File(prefixHeaderDir, "prefix-headers.h");
-                                internalSourceSet.setPrefixHeaderFile(prefixHeaderFile);
+                                dependentSourceSet.setPrefixHeaderFile(prefixHeaderFile);
                             }
                         }
                     });
@@ -225,14 +229,13 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         @Mutate
         void configurePrefixHeaderGenerationTasks(final TaskContainer tasks, ModelMap<NativeComponentSpec> nativeComponents) {
             for (final NativeComponentSpec nativeComponentSpec : nativeComponents.values()) {
-                for (final DependentSourceSet dependentSourceSet : nativeComponentSpec.getSources().withType(DependentSourceSet.class).values()) {
-                    final DependentSourceSetInternal internalSourceSet = (DependentSourceSetInternal) dependentSourceSet;
-                    if (internalSourceSet.getPrefixHeaderFile() != null) {
+                for (final DependentSourceSetInternal dependentSourceSet : nativeComponentSpec.getSources().withType(DependentSourceSetInternal.class).values()) {
+                    if (dependentSourceSet.getPrefixHeaderFile() != null) {
                         String taskName = String.format("generate%s%sPrefixHeaderFile", StringUtils.capitalize(nativeComponentSpec.getName()), StringUtils.capitalize(dependentSourceSet.getName()));
                         tasks.create(taskName, PrefixHeaderFileGenerateTask.class, new Action<PrefixHeaderFileGenerateTask>() {
                             @Override
                             public void execute(PrefixHeaderFileGenerateTask prefixHeaderFileGenerateTask) {
-                                prefixHeaderFileGenerateTask.setPrefixHeaderFile(internalSourceSet.getPrefixHeaderFile());
+                                prefixHeaderFileGenerateTask.setPrefixHeaderFile(dependentSourceSet.getPrefixHeaderFile());
                                 prefixHeaderFileGenerateTask.setHeader(dependentSourceSet.getPreCompiledHeader());
                             }
                         });
@@ -252,7 +255,7 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
                             if (dependentSourceSet.getPreCompiledHeader() != null) {
                                 nativeBinarySpec.getPrefixFileToPCH().put(((DependentSourceSetInternal) dependentSourceSet).getPrefixHeaderFile(), new PreCompiledHeader());
                                 final SourceTransformTaskConfig pchTransformTaskConfig = transform.getPchTransformTask();
-                                String pchTaskName = String.format("%s%s%sPreCompiledHeader", pchTransformTaskConfig.getTaskPrefix(), StringUtils.capitalize(nativeBinarySpec.getName()), StringUtils.capitalize(dependentSourceSet.getName()));
+                                String pchTaskName = String.format("%s%s%sPreCompiledHeader", pchTransformTaskConfig.getTaskPrefix(), StringUtils.capitalize(nativeBinarySpec.getProjectScopedName()), StringUtils.capitalize(dependentSourceSet.getName()));
                                 Task pchTask = tasks.create(pchTaskName, pchTransformTaskConfig.getTaskType(), new Action<DefaultTask>() {
                                     @Override
                                     public void execute(DefaultTask task) {
@@ -275,19 +278,19 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         }
 
         @BinaryTasks
-        public void createTasks(ModelMap<Task> tasks, final SharedLibraryBinarySpecInternal binary) {
+        public void sharedLibraryTasks(ModelMap<Task> tasks, final SharedLibraryBinarySpecInternal binary) {
             String taskName = binary.getNamingScheme().getTaskName("link");
             tasks.create(taskName, LinkSharedLibrary.class, new Action<LinkSharedLibrary>() {
                 @Override
                 public void execute(LinkSharedLibrary linkTask) {
-                    linkTask.setDescription("Links " + binary.getName());
+                    linkTask.setDescription("Links " + binary.getDisplayName());
                     linkTask.setToolChain(binary.getToolChain());
                     linkTask.setTargetPlatform(binary.getTargetPlatform());
                     linkTask.setOutputFile(binary.getSharedLibraryFile());
                     linkTask.setInstallName(binary.getSharedLibraryFile().getName());
                     linkTask.setLinkerArgs(binary.getLinker().getArgs());
 
-                    linkTask.lib(new BinaryLibs(binary) {
+                    linkTask.lib(new NativeComponents.BinaryLibs(binary) {
                         @Override
                         protected FileCollection getFiles(NativeDependencySet nativeDependencySet) {
                             return nativeDependencySet.getLinkFiles();
@@ -298,12 +301,12 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         }
 
         @BinaryTasks
-        public void createTasks(ModelMap<Task> tasks, final StaticLibraryBinarySpecInternal binary) {
+        public void staticLibraryTasks(ModelMap<Task> tasks, final StaticLibraryBinarySpecInternal binary) {
             String taskName = binary.getNamingScheme().getTaskName("create");
             tasks.create(taskName, CreateStaticLibrary.class, new Action<CreateStaticLibrary>() {
                 @Override
                 public void execute(CreateStaticLibrary task) {
-                    task.setDescription("Creates " + binary.getName());
+                    task.setDescription("Creates " + binary.getDisplayName());
                     task.setToolChain(binary.getToolChain());
                     task.setTargetPlatform(binary.getTargetPlatform());
                     task.setOutputFile(binary.getStaticLibraryFile());
@@ -313,94 +316,20 @@ public class NativeComponentModelPlugin implements Plugin<ProjectInternal> {
         }
 
         @BinaryTasks
-        public void createTasks(ModelMap<Task> tasks, final NativeExecutableBinarySpecInternal executableBinary) {
-            createExecutableTask(tasks, executableBinary, executableBinary.getExecutable().getFile());
-        }
-
-        @BinaryTasks
-        public void createTasks(ModelMap<Task> tasks, final NativeTestSuiteBinarySpecInternal testSuiteBinary) {
-            createExecutableTask(tasks, testSuiteBinary, testSuiteBinary.getExecutableFile());
-        }
-
-        private void createExecutableTask(ModelMap<Task> tasks, final NativeBinarySpecInternal binary, final File executableFile) {
-            String taskName = binary.getNamingScheme().getTaskName("link");
-            tasks.create(taskName, LinkExecutable.class, new Action<LinkExecutable>() {
-                @Override
-                public void execute(LinkExecutable linkTask) {
-                    linkTask.setDescription("Links " + binary.getName());
-                    linkTask.setToolChain(binary.getToolChain());
-                    linkTask.setTargetPlatform(binary.getTargetPlatform());
-                    linkTask.setOutputFile(executableFile);
-                    linkTask.setLinkerArgs(binary.getLinker().getArgs());
-
-                    linkTask.lib(new BinaryLibs(binary) {
-                        @Override
-                        protected FileCollection getFiles(NativeDependencySet nativeDependencySet) {
-                            return nativeDependencySet.getLinkFiles();
-                        }
-                    });
-                }
-            });
+        public void executableTasks(ModelMap<Task> tasks, final NativeExecutableBinarySpecInternal executableBinary) {
+            NativeComponents.createExecutableTask(executableBinary, executableBinary.getExecutable().getFile());
         }
 
         /**
-         * Can't use @BinaryTasks because the binary is not _built-by_ the install task, but it is associated with it.
-         * Rule is called multiple times, so need to check for task existence before creating.
+         * Can't use @BinaryTasks because the binary is not _built-by_ the install task, but it is associated with it. Rule is called multiple times, so need to check for task existence before
+         * creating.
          */
         @Defaults
-        void createInstallTasks(TaskContainer tasks, ModelMap<NativeBinarySpecInternal> binaries, @Path("buildDir") File buildDir) {
-            // There's no common API for NativeExecutableBinarySpec and NativeTestSuiteBinarySpec
+        void createInstallTasks(ModelMap<Task> tasks, ModelMap<NativeBinarySpecInternal> binaries) {
             for (NativeExecutableBinarySpecInternal binary : binaries.withType(NativeExecutableBinarySpecInternal.class).values()) {
-                Task installTask = createInstallTask(tasks, binary, binary.getInstallation(), binary.getExecutable(), buildDir, binary.getNamingScheme());
-                binary.getTasks().add(installTask);
-            }
-            for (NativeTestSuiteBinarySpecInternal binary : binaries.withType(NativeTestSuiteBinarySpecInternal.class).values()) {
-                Task installTask = createInstallTask(tasks, binary, binary.getInstallation(), binary.getExecutable(), buildDir, binary.getNamingScheme());
-                binary.getTasks().add(installTask);
+                NativeComponents.createInstallTask(binary, binary.getInstallation(), binary.getExecutable(), binary.getNamingScheme());
             }
         }
-
-        private Task createInstallTask(TaskContainer tasks, final NativeBinarySpecInternal binary, final NativeInstallationSpec installation, final NativeExecutableFileSpec executable, final File buildDirectory, final BinaryNamingScheme namingScheme) {
-            return tasks.create(namingScheme.getTaskName("install"), InstallExecutable.class, new Action<InstallExecutable>() {
-                @Override
-                public void execute(InstallExecutable installTask) {
-                    installTask.setDescription("Installs a development image of " + namingScheme.getBaseName());
-                    installTask.setGroup(LifecycleBasePlugin.BUILD_GROUP);
-                    installTask.setToolChain(executable.getToolChain());
-                    installTask.setExecutable(executable.getFile());
-                    installTask.setDestinationDir(installation.getDirectory());
-                    //TODO:HH wire binary libs via executable
-                    installTask.lib(new BinaryLibs(binary) {
-                        @Override
-                        protected FileCollection getFiles(NativeDependencySet nativeDependencySet) {
-                            return nativeDependencySet.getRuntimeFiles();
-                        }
-                    });
-
-                    //TODO:HH installTask.dependsOn(executable)
-                    installTask.dependsOn(binary);
-                }
-            });
-        }
-    }
-
-    private abstract static class BinaryLibs implements Callable<List<FileCollection>> {
-        private final NativeBinarySpec binary;
-
-        public BinaryLibs(NativeBinarySpec binary) {
-            this.binary = binary;
-        }
-
-        @Override
-        public List<FileCollection> call() throws Exception {
-            List<FileCollection> runtimeFiles = Lists.newArrayList();
-            for (NativeDependencySet nativeDependencySet : binary.getLibs()) {
-                runtimeFiles.add(getFiles(nativeDependencySet));
-            }
-            return runtimeFiles;
-        }
-
-        protected abstract FileCollection getFiles(NativeDependencySet nativeDependencySet);
     }
 
     private static class DefaultRepositories extends DefaultPolymorphicDomainObjectContainer<ArtifactRepository> implements Repositories {
