@@ -16,6 +16,7 @@
 
 package org.gradle.play.internal.twirl;
 
+import org.gradle.language.twirl.TwirlImports;
 import org.gradle.scala.internal.reflect.ScalaMethod;
 import org.gradle.scala.internal.reflect.ScalaReflectionUtil;
 
@@ -26,15 +27,22 @@ import java.util.Arrays;
 class TwirlCompilerAdapterV22X implements VersionedTwirlCompilerAdapter {
     private static final Iterable<String> SHARED_PACKAGES = Arrays.asList("play.templates");
 
+    // Based on https://github.com/playframework/playframework/blob/2.2.6/framework/src/sbt-plugin/src/main/scala/PlayKeys.scala
     private static final String DEFAULT_JAVA_IMPORTS =
               "import play.api.templates._;"
             + "import play.api.templates.PlayMagic._;"
             + "import models._;"
             + "import controllers._;"
+            + "import java.lang._;"
+            + "import java.util._;"
+            + "import scala.collection.JavaConversions._;"
+            + "import scala.collection.JavaConverters._;"
             + "import play.api.i18n._;"
-            + "import play.api.mvc._;"
-            + "import play.api.mvc._;"
-            + "import play.api.data._;"
+            + "import play.core.j.PlayMagicForJava._;"
+            + "import play.mvc._;"
+            + "import play.data._;"
+            + "import play.api.data.Field;"
+            + "import play.mvc.Http.Context.Implicit._;"
             + "import views.html._;";
 
     private static final String DEFAULT_SCALA_IMPORTS =
@@ -44,7 +52,6 @@ class TwirlCompilerAdapterV22X implements VersionedTwirlCompilerAdapter {
             + "import controllers._;"
             + "import play.api.i18n._;"
             + "import play.api.mvc._;"
-            + "import play.api.mvc._;"
             + "import play.api.data._;"
             + "import views.html._;";
 
@@ -70,13 +77,13 @@ class TwirlCompilerAdapterV22X implements VersionedTwirlCompilerAdapter {
     }
 
     @Override
-    public Object[] createCompileParameters(ClassLoader cl, File file, File sourceDirectory, File destinationDirectory, boolean javaProject) throws ClassNotFoundException, NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {
+    public Object[] createCompileParameters(ClassLoader cl, File file, File sourceDirectory, File destinationDirectory, TwirlImports defaultImports) throws ClassNotFoundException, NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {
         return new Object[] {
                 file,
                 sourceDirectory,
                 destinationDirectory,
                 "play.api.templates.HtmlFormat",
-                javaProject ? DEFAULT_JAVA_IMPORTS : DEFAULT_SCALA_IMPORTS
+                defaultImports == TwirlImports.JAVA ? DEFAULT_JAVA_IMPORTS : DEFAULT_SCALA_IMPORTS
         };
     }
 
