@@ -15,7 +15,6 @@
  */
 package org.gradle.play.plugins;
 
-import org.apache.commons.lang.StringUtils;
 import org.gradle.api.*;
 import org.gradle.api.artifacts.Dependency;
 import org.gradle.api.internal.artifacts.publish.DefaultPublishArtifact;
@@ -23,33 +22,24 @@ import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.file.copy.CopySpecInternal;
 import org.gradle.api.internal.project.ProjectIdentifier;
 import org.gradle.api.plugins.ExtensionContainer;
-import org.gradle.api.tasks.scala.IncrementalCompileOptions;
-import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.jvm.internal.JvmAssembly;
 import org.gradle.jvm.tasks.Jar;
-import org.gradle.language.base.LanguageSourceSet;
-import org.gradle.language.base.sources.BaseLanguageSourceSet;
-import org.gradle.language.java.JavaSourceSet;
 import org.gradle.language.java.plugins.JavaLanguagePlugin;
-import org.gradle.language.jvm.JvmResourceSet;
-import org.gradle.language.routes.RoutesSourceSet;
-import org.gradle.language.routes.internal.DefaultRoutesSourceSet;
+import org.gradle.language.javascript.JavaScriptSourceSet;
 import org.gradle.language.scala.ScalaLanguageSourceSet;
-import org.gradle.language.scala.internal.DefaultScalaLanguageSourceSet;
 import org.gradle.language.scala.plugins.ScalaLanguagePlugin;
 import org.gradle.language.scala.tasks.PlatformScalaCompile;
-import org.gradle.language.twirl.TwirlSourceSet;
-import org.gradle.language.twirl.internal.DefaultTwirlSourceSet;
 import org.gradle.model.*;
+import org.gradle.model.internal.core.Hidden;
 import org.gradle.model.internal.registry.ModelRegistry;
-import org.gradle.model.internal.type.ModelType;
 import org.gradle.platform.base.*;
 import org.gradle.platform.base.internal.DefaultPlatformRequirement;
 import org.gradle.platform.base.internal.PlatformRequirement;
 import org.gradle.platform.base.internal.PlatformResolvers;
-import org.gradle.play.JvmClasses;
 import org.gradle.play.PlayApplicationBinarySpec;
 import org.gradle.play.PlayApplicationSpec;
+import org.gradle.play.PlayPlatformAwareComponentSpec;
 import org.gradle.play.PublicAssets;
 import org.gradle.play.internal.*;
 import org.gradle.play.internal.platform.PlayPlatformInternal;
@@ -57,15 +47,15 @@ import org.gradle.play.internal.run.PlayApplicationRunnerFactory;
 import org.gradle.play.internal.toolchain.PlayToolChainInternal;
 import org.gradle.play.platform.PlayPlatform;
 import org.gradle.play.tasks.PlayRun;
-import org.gradle.play.tasks.RoutesCompile;
-import org.gradle.play.tasks.TwirlCompile;
 import org.gradle.util.VersionNumber;
 
 import javax.inject.Inject;
 import java.io.File;
-import java.util.ArrayList;
 import java.util.Date;
 
+import static org.gradle.model.internal.core.ModelNodes.withType;
+import static org.gradle.model.internal.core.NodePredicate.allDescendants;
+
 /**
  * Plugin for Play Framework component support. Registers the {@link org.gradle.play.PlayApplicationSpec} component type for the components container.
  */
@@ -84,9 +74,12 @@ public class PlayApplicationPlugin implements Plugin<Project> {
     public void apply(Project project) {
         project.getPluginManager().apply(JavaLanguagePlugin.class);
         project.getPluginManager().apply(ScalaLanguagePlugin.class);
+        project.getPluginManager().apply(PlayTwirlPlugin.class);
+        project.getPluginManager().apply(PlayRoutesPlugin.class);
+
         project.getExtensions().create("playConfigurations", PlayPluginConfigurations.class, project.getConfigurations(), project.getDependencies());
 
-        modelRegistry.getRoot().applyToAllLinksTransitive(ModelType.of(PlayApplicationSpec.class), PlaySourceSetRules.class);
+        modelRegistry.getRoot().applyTo(allDescendants(withType(PlayApplicationSpec.class)), PlaySourceSetRules.class);
     }
 
     @SuppressWarnings("UnusedDeclaration")
@@ -101,14 +94,20 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             return serviceRegistry.get(PlayToolChainInternal.class);
         }
 
-        @Model
+        @Hidden @Model
         FileResolver fileResolver(ServiceRegistry serviceRegistry) {
             return serviceRegistry.get(FileResolver.class);
         }
 
         @ComponentType
-        void register(ComponentTypeBuilder<PlayApplicationSpec> builder) {
-            builder.defaultImplementation(DefaultPlayApplicationSpec.class);
+        void registerPlayPlatformAwareComponentSpecType(ComponentTypeBuilder<PlayPlatformAwareComponentSpec> builder) {
+            builder.defaultImplementation(DefaultPlayPlatformAwareComponentSpec.class);
+            builder.internalView(PlayPlatformAwareComponentSpecInternal.class);
+        }
+
+        @ComponentType
+        void registerPlayApplicationSpecType(ComponentTypeBuilder<PlayApplicationSpec> builder) {
+            builder.internalView(PlayApplicationSpecInternal.class);
         }
 
         @Mutate
@@ -127,18 +126,6 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             builder.internalView(PlayApplicationBinarySpecInternal.class);
         }
 
-        @LanguageType
-        void registerTwirlLanguageType(LanguageTypeBuilder<TwirlSourceSet> builder) {
-            builder.setLanguageName("twirl");
-            builder.defaultImplementation(DefaultTwirlSourceSet.class);
-        }
-
-        @LanguageType
-        void registerRoutesLanguageType(LanguageTypeBuilder<RoutesSourceSet> builder) {
-            builder.setLanguageName("routes");
-            builder.defaultImplementation(DefaultRoutesSourceSet.class);
-        }
-
         @Validate
         void failOnMultiplePlayComponents(ModelMap<PlayApplicationSpec> container) {
             if (container.size() >= 2) {
@@ -148,10 +135,9 @@ public class PlayApplicationPlugin implements Plugin<Project> {
 
         @Validate
         void failOnMultipleTargetPlatforms(ModelMap<PlayApplicationSpec> playApplications) {
-            playApplications.afterEach(new Action<PlayApplicationSpec>() {
-                public void execute(PlayApplicationSpec playApplication) {
-                    PlayApplicationSpecInternal playApplicationInternal = (PlayApplicationSpecInternal) playApplication;
-                    if (playApplicationInternal.getTargetPlatforms().size() > 1) {
+            playApplications.withType(PlayPlatformAwareComponentSpecInternal.class).afterEach(new Action<PlayPlatformAwareComponentSpecInternal>() {
+                public void execute(PlayPlatformAwareComponentSpecInternal playApplication) {
+                    if (playApplication.getTargetPlatforms().size() > 1) {
                         throw new GradleException("Multiple target platforms for 'PlayApplicationSpec' is not (yet) supported.");
                     }
                 }
@@ -176,19 +162,14 @@ public class PlayApplicationPlugin implements Plugin<Project> {
         }
 
         @ComponentBinaries
-        void createBinaries(ModelMap<PlayApplicationBinarySpec> binaries, final PlayApplicationSpec componentSpec,
-                            final PlatformResolvers platforms, final PlayToolChainInternal playToolChainInternal, final PlayPluginConfigurations configurations, final ServiceRegistry serviceRegistry,
+        void createBinaries(ModelMap<PlayApplicationBinarySpec> binaries, final PlayApplicationSpecInternal componentSpec,
+                            final PlatformResolvers platforms, final PlayToolChainInternal playToolChainInternal, final PlayPluginConfigurations configurations,
                             @Path("buildDir") final File buildDir, final ProjectIdentifier projectIdentifier) {
 
-            final FileResolver fileResolver = serviceRegistry.get(FileResolver.class);
-            final Instantiator instantiator = serviceRegistry.get(Instantiator.class);
-            final String binaryName = String.format("%sBinary", componentSpec.getName());
-
-            binaries.create(binaryName, new Action<PlayApplicationBinarySpec>() {
+            binaries.create("binary", new Action<PlayApplicationBinarySpec>() {
                 public void execute(PlayApplicationBinarySpec playBinary) {
                     PlayApplicationBinarySpecInternal playBinaryInternal = (PlayApplicationBinarySpecInternal) playBinary;
-                    playBinaryInternal.setApplication(componentSpec);
-                    final File binaryBuildDir = new File(buildDir, binaryName);
+                    final File binaryBuildDir = new File(buildDir, playBinaryInternal.getProjectScopedName());
 
                     final PlayPlatform chosenPlatform = resolveTargetPlatform(componentSpec, platforms);
                     initialiseConfigurations(configurations, chosenPlatform);
@@ -204,15 +185,9 @@ public class PlayApplicationPlugin implements Plugin<Project> {
                     configurations.getPlay().addArtifact(new DefaultPublishArtifact(projectIdentifier.getName(), "jar", "jar", null, new Date(), mainJar, playBinaryInternal));
                     configurations.getPlay().addArtifact(new DefaultPublishArtifact(projectIdentifier.getName(), "jar", "jar", "assets", new Date(), assetsJar, playBinaryInternal));
 
-                    JvmClasses classes = playBinary.getClasses();
-                    classes.setClassesDir(new File(binaryBuildDir, "classes"));
-
-                    ModelMap<JvmResourceSet> jvmResourceSets = componentSpec.getSources().withType(JvmResourceSet.class);
-                    for (JvmResourceSet jvmResourceSet : jvmResourceSets.values()) {
-                        for (File resourceDir : jvmResourceSet.getSource()) {
-                            classes.addResourceDir(resourceDir);
-                        }
-                    }
+                    JvmAssembly jvmAssembly = ((PlayApplicationBinarySpecInternal) playBinary).getAssembly();
+                    jvmAssembly.getClassDirectories().add(new File(binaryBuildDir, "classes"));
+                    jvmAssembly.getResourceDirectories().add(new File(binaryBuildDir, "resources"));
 
                     PublicAssets assets = playBinary.getAssets();
                     assets.addAssetDir(new File(projectIdentifier.getProjectDir(), "public"));
@@ -222,8 +197,23 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             });
         }
 
-        private PlayPlatform resolveTargetPlatform(PlayApplicationSpec componentSpec, final PlatformResolvers platforms) {
-            PlatformRequirement targetPlatform = getTargetPlatform((PlayApplicationSpecInternal) componentSpec);
+        @Mutate
+        void generatedSourcesAreInputs(ModelMap<PlayApplicationBinarySpecInternal> binaries, final ServiceRegistry serviceRegistry) {
+            binaries.afterEach(new Action<PlayApplicationBinarySpecInternal>() {
+                @Override
+                public void execute(PlayApplicationBinarySpecInternal playApplicationBinarySpec) {
+                    for (ScalaLanguageSourceSet generatedSources : playApplicationBinarySpec.getGeneratedScala().values()) {
+                        playApplicationBinarySpec.getInputs().add(generatedSources);
+                    }
+                    for (JavaScriptSourceSet generatedSources : playApplicationBinarySpec.getGeneratedJavaScript().values()) {
+                        playApplicationBinarySpec.getInputs().add(generatedSources);
+                    }
+                }
+            });
+        }
+
+        private PlayPlatform resolveTargetPlatform(PlayApplicationSpecInternal componentSpec, final PlatformResolvers platforms) {
+            PlatformRequirement targetPlatform = getTargetPlatform(componentSpec);
             return platforms.resolve(PlayPlatform.class, targetPlatform);
         }
 
@@ -252,128 +242,34 @@ public class PlayApplicationPlugin implements Plugin<Project> {
             }
         }
 
-        @Mutate
-        void createGeneratedScalaSourceSets(ModelMap<PlayApplicationBinarySpec> binaries, final ServiceRegistry serviceRegistry) {
-            createGeneratedScalaSourceSetsForType(TwirlSourceSet.class, binaries, serviceRegistry);
-            createGeneratedScalaSourceSetsForType(RoutesSourceSet.class, binaries, serviceRegistry);
-        }
-
-        void createGeneratedScalaSourceSetsForType(final Class<? extends LanguageSourceSet> languageSourceSetType, ModelMap<PlayApplicationBinarySpec> binaries, ServiceRegistry serviceRegistry) {
-            final FileResolver fileResolver = serviceRegistry.get(FileResolver.class);
-            final Instantiator instantiator = serviceRegistry.get(Instantiator.class);
-            binaries.all(new Action<PlayApplicationBinarySpec>() {
-                @Override
-                public void execute(PlayApplicationBinarySpec playApplicationBinarySpec) {
-                    for (LanguageSourceSet languageSourceSet : playApplicationBinarySpec.getInputs().withType(languageSourceSetType)) {
-                        String name = String.format("%sScalaSources", languageSourceSet.getName());
-                        ScalaLanguageSourceSet twirlScalaSources = BaseLanguageSourceSet.create(DefaultScalaLanguageSourceSet.class, name, playApplicationBinarySpec.getName(), fileResolver, instantiator);
-                        playApplicationBinarySpec.getGeneratedScala().put(languageSourceSet, twirlScalaSources);
-                    }
-                }
-            });
-        }
-
-        @BinaryTasks
-        void createTwirlCompileTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary, ServiceRegistry serviceRegistry, @Path("buildDir") final File buildDir) {
-            for (final TwirlSourceSet twirlSourceSet : binary.getInputs().withType(TwirlSourceSet.class)) {
-                final String twirlCompileTaskName = String.format("compile%s%s", StringUtils.capitalize(binary.getName()), StringUtils.capitalize(twirlSourceSet.getName()));
-                final File twirlCompileOutputDirectory = srcOutputDirectory(buildDir, binary, twirlCompileTaskName);
-
-                tasks.create(twirlCompileTaskName, TwirlCompile.class, new Action<TwirlCompile>() {
-                    public void execute(TwirlCompile twirlCompile) {
-                        twirlCompile.setDescription("Compiles twirl templates for the '" + twirlSourceSet.getName() + "' source set.");
-                        twirlCompile.setPlatform(binary.getTargetPlatform());
-                        twirlCompile.setSource(twirlSourceSet.getSource());
-                        twirlCompile.setOutputDirectory(twirlCompileOutputDirectory);
-
-                        ScalaLanguageSourceSet twirlScalaSources = binary.getGeneratedScala().get(twirlSourceSet);
-                        twirlScalaSources.getSource().srcDir(twirlCompileOutputDirectory);
-                        twirlScalaSources.builtBy(twirlCompile);
-                    }
-                });
-            }
-        }
-
-        @BinaryTasks
-        void createRoutesCompileTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary, ServiceRegistry serviceRegistry, @Path("buildDir") final File buildDir) {
-            for (final RoutesSourceSet routesSourceSet : binary.getInputs().withType(RoutesSourceSet.class)) {
-                final String routesCompileTaskName = String.format("compile%s%s", StringUtils.capitalize(binary.getName()), StringUtils.capitalize(routesSourceSet.getName()));
-                final File routesCompilerOutputDirectory = srcOutputDirectory(buildDir, binary, routesCompileTaskName);
-
-                tasks.create(routesCompileTaskName, RoutesCompile.class, new Action<RoutesCompile>() {
-                    public void execute(RoutesCompile routesCompile) {
-                        routesCompile.setDescription("Generates routes for the '" + routesSourceSet.getName() + "' source set.");
-                        routesCompile.setPlatform(binary.getTargetPlatform());
-                        routesCompile.setAdditionalImports(new ArrayList<String>());
-                        routesCompile.setSource(routesSourceSet.getSource());
-                        routesCompile.setOutputDirectory(routesCompilerOutputDirectory);
-                        routesCompile.setInjectedRoutesGenerator(binary.getApplication().getInjectedRoutesGenerator());
-
-                        ScalaLanguageSourceSet routesScalaSources = binary.getGeneratedScala().get(routesSourceSet);
-                        routesScalaSources.getSource().srcDir(routesCompilerOutputDirectory);
-                        routesScalaSources.builtBy(routesCompile);
-                    }
-                });
-            }
-        }
-
         @BinaryTasks
         void createScalaCompileTask(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary, @Path("buildDir") final File buildDir) {
-            final String scalaCompileTaskName = String.format("compile%s%s", StringUtils.capitalize(binary.getName()), "Scala");
-            tasks.create(scalaCompileTaskName, PlatformScalaCompile.class, new Action<PlatformScalaCompile>() {
+            tasks.withType(PlatformScalaCompile.class).afterEach(new Action<PlatformScalaCompile>() {
+                @Override
                 public void execute(PlatformScalaCompile scalaCompile) {
-                    scalaCompile.setDescription("Compiles all scala and java source sets for the '" + binary.getName() + "' binary.");
-
-                    scalaCompile.setDestinationDir(binary.getClasses().getClassesDir());
-                    scalaCompile.setPlatform(binary.getTargetPlatform().getScalaPlatform());
-                    //infer scala classpath
-                    String targetCompatibility = binary.getTargetPlatform().getJavaPlatform().getTargetCompatibility().getMajorVersion();
-                    scalaCompile.setSourceCompatibility(targetCompatibility);
-                    scalaCompile.setTargetCompatibility(targetCompatibility);
-
-                    IncrementalCompileOptions incrementalOptions = scalaCompile.getScalaCompileOptions().getIncrementalOptions();
-                    incrementalOptions.setAnalysisFile(new File(buildDir, String.format("tmp/scala/compilerAnalysis/%s.analysis", scalaCompileTaskName)));
-
-                    for (LanguageSourceSet appSources : binary.getInputs().withType(ScalaLanguageSourceSet.class)) {
-                        scalaCompile.source(appSources.getSource());
-                        scalaCompile.dependsOn(appSources);
-                    }
-
-                    for (LanguageSourceSet appSources : binary.getInputs().withType(JavaSourceSet.class)) {
-                        scalaCompile.source(appSources.getSource());
-                        scalaCompile.dependsOn(appSources);
-                    }
-
-                    for (LanguageSourceSet generatedSourceSet : binary.getGeneratedScala().values()) {
-                        scalaCompile.source(generatedSourceSet.getSource());
-                        scalaCompile.dependsOn(generatedSourceSet);
-                    }
-
                     scalaCompile.setClasspath(((PlayApplicationBinarySpecInternal) binary).getClasspath());
-
-                    binary.getClasses().builtBy(scalaCompile);
                 }
             });
         }
 
         @BinaryTasks
-        void createJarTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpec binary) {
-            String jarTaskName = String.format("create%sJar", StringUtils.capitalize(binary.getName()));
+        void createJarTasks(ModelMap<Task> tasks, final PlayApplicationBinarySpecInternal binary) {
+            String jarTaskName = binary.getTasks().taskName("create", "Jar");
             tasks.create(jarTaskName, Jar.class, new Action<Jar>() {
                 public void execute(Jar jar) {
-                    jar.setDescription("Assembles the application jar for the '" + binary.getName() + "' binary.");
+                    jar.setDescription("Assembles the application jar for the " + binary.getDisplayName() + ".");
                     jar.setDestinationDir(binary.getJarFile().getParentFile());
                     jar.setArchiveName(binary.getJarFile().getName());
-                    jar.from(binary.getClasses().getClassesDir());
-                    jar.from(binary.getClasses().getResourceDirs());
-                    jar.dependsOn(binary.getClasses());
+                    jar.from(binary.getAssembly().getClassDirectories());
+                    jar.from(binary.getAssembly().getResourceDirectories());
+                    jar.dependsOn(binary.getAssembly());
                 }
             });
 
-            String assetsJarTaskName = String.format("create%sAssetsJar", StringUtils.capitalize(binary.getName()));
+            String assetsJarTaskName = binary.getTasks().taskName("create", "assetsJar");
             tasks.create(assetsJarTaskName, Jar.class, new Action<Jar>() {
                 public void execute(Jar jar) {
-                    jar.setDescription("Assembles the assets jar for the '" + binary.getName() + "' binary.");
+                    jar.setDescription("Assembles the assets jar for the " + binary.getDisplayName() + ".");
                     jar.setDestinationDir(binary.getAssetsJarFile().getParentFile());
                     jar.setArchiveName(binary.getAssetsJarFile().getName());
                     jar.setClassifier("assets");
@@ -389,7 +285,7 @@ public class PlayApplicationPlugin implements Plugin<Project> {
         void createPlayRunTask(ModelMap<Task> tasks, ModelMap<PlayApplicationBinarySpecInternal> playBinaries, final ServiceRegistry serviceRegistry, final PlayPluginConfigurations configurations, ProjectIdentifier projectIdentifier, final PlayToolChainInternal playToolChain) {
 
             for (final PlayApplicationBinarySpecInternal binary : playBinaries) {
-                String runTaskName = String.format("run%s", StringUtils.capitalize(binary.getName()));
+                String runTaskName = binary.getTasks().taskName("run");
 
                 tasks.create(runTaskName, PlayRun.class, new Action<PlayRun>() {
                     public void execute(PlayRun playRun) {
@@ -407,9 +303,5 @@ public class PlayApplicationPlugin implements Plugin<Project> {
                 });
             }
         }
-
-        private File srcOutputDirectory(File buildDir, PlayApplicationBinarySpec binary, String taskName) {
-            return new File(buildDir, String.format("%s/src/%s", binary.getName(), taskName));
-        }
     }
 }
