@@ -48,14 +48,14 @@ import org.gradle.language.base.plugins.LanguageBasePlugin;
 import org.gradle.language.base.plugins.LifecycleBasePlugin;
 import org.gradle.language.jvm.JvmResourceSet;
 import org.gradle.language.jvm.tasks.ProcessResources;
-import org.gradle.model.ModelMap;
 import org.gradle.model.Mutate;
-import org.gradle.model.Path;
 import org.gradle.model.RuleSource;
-import org.gradle.model.internal.core.ModelCreators;
 import org.gradle.model.internal.core.ModelReference;
+import org.gradle.model.internal.core.ModelRegistrations;
 import org.gradle.model.internal.registry.ModelRegistry;
-import org.gradle.platform.base.BinarySpec;
+import org.gradle.platform.base.BinaryContainer;
+import org.gradle.platform.base.internal.BinarySpecInternal;
+import org.gradle.platform.base.plugins.BinaryBasePlugin;
 import org.gradle.util.WrapUtil;
 
 import javax.inject.Inject;
@@ -92,6 +92,7 @@ public class JavaBasePlugin implements Plugin<ProjectInternal> {
         project.getPluginManager().apply(BasePlugin.class);
         project.getPluginManager().apply(ReportingBasePlugin.class);
         project.getPluginManager().apply(LanguageBasePlugin.class);
+        project.getPluginManager().apply(BinaryBasePlugin.class);
 
         JavaPluginConvention javaConvention = new JavaPluginConvention(project, instantiator);
         project.getConvention().getPlugins().put("java", javaConvention);
@@ -99,11 +100,10 @@ public class JavaBasePlugin implements Plugin<ProjectInternal> {
         configureCompileDefaults(project, javaConvention);
         BridgedBinaries binaries = configureSourceSetDefaults(javaConvention);
 
-        modelRegistry.createOrReplace(ModelCreators.bridgedInstance(ModelReference.of("bridgedBinaries", BridgedBinaries.class), binaries)
-                .descriptor("JavaBasePlugin.apply()")
-                .ephemeral(true)
-                .hidden(true)
-                .build());
+        modelRegistry.register(ModelRegistrations.bridgedInstance(ModelReference.of("bridgedBinaries", BridgedBinaries.class), binaries)
+            .descriptor("JavaBasePlugin.apply()")
+            .hidden(true)
+            .build());
 
         configureJavaDoc(project, javaConvention);
         configureTest(project, javaConvention);
@@ -395,16 +395,16 @@ public class JavaBasePlugin implements Plugin<ProjectInternal> {
 
     static class Rules extends RuleSource {
         @Mutate
-        void attachBridgedSourceSets(ProjectSourceSet projectSourceSet, @Path("bridgedBinaries") BridgedBinaries bridgedBinaries) {
+        void attachBridgedSourceSets(ProjectSourceSet projectSourceSet, BridgedBinaries bridgedBinaries) {
             for (ClassDirectoryBinarySpecInternal binary : bridgedBinaries.binaries) {
                 projectSourceSet.addAll(binary.getInputs());
             }
         }
 
         @Mutate
-        void attachBridgedBinaries(ModelMap<BinarySpec> binaries, @Path("bridgedBinaries") BridgedBinaries bridgedBinaries) {
-            for (BinarySpec binary : bridgedBinaries.binaries) {
-                binaries.put(binary.getName(), binary);
+        void attachBridgedBinaries(BinaryContainer binaries, BridgedBinaries bridgedBinaries) {
+            for (BinarySpecInternal binary : bridgedBinaries.binaries) {
+                binaries.put(binary.getProjectScopedName(), binary);
             }
         }
     }
