@@ -33,7 +33,9 @@ import org.gradle.tooling.events.ProgressListener;
 import org.gradle.tooling.events.task.*;
 import org.gradle.tooling.internal.consumer.DefaultBuildLauncher;
 import org.gradle.tooling.internal.consumer.DefaultGradleConnector;
+import org.gradle.tooling.model.build.BuildEnvironment;
 import org.gradle.util.CollectionUtils;
+import org.gradle.util.GradleVersion;
 import org.gradle.wrapper.GradleUserHomeLookup;
 
 import java.io.ByteArrayOutputStream;
@@ -82,9 +84,11 @@ public class ToolingApiGradleExecutor implements GradleExecutor {
         );
 
         ProjectConnection connection = null;
+        GradleVersion targetGradleVersion = null;
 
         try {
             connection = gradleConnector.connect();
+            targetGradleVersion = determineTargetGradleVersion(connection);
             DefaultBuildLauncher launcher = (DefaultBuildLauncher) connection.newBuild();
 
             launcher.setStandardOutput(new NoCloseOutputStream(teeOutput(syncOutput, parameters.getStandardOutput())));
@@ -101,7 +105,7 @@ public class ToolingApiGradleExecutor implements GradleExecutor {
         } catch (UnsupportedVersionException e) {
             throw new InvalidRunnerConfigurationException("The build could not be executed due to a feature not being supported by the target Gradle version", e);
         } catch (BuildException t) {
-            return new GradleExecutionResult(output.toString(), tasks, t);
+            return new GradleExecutionResult(new BuildOperationParameters(targetGradleVersion, parameters.isEmbedded()), output.toString(), tasks, t);
         } catch (GradleConnectionException t) {
             StringBuilder message = new StringBuilder("An error occurred executing build with ");
             if (parameters.getBuildArgs().isEmpty()) {
@@ -128,7 +132,12 @@ public class ToolingApiGradleExecutor implements GradleExecutor {
             }
         }
 
-        return new GradleExecutionResult(output.toString(), tasks);
+        return new GradleExecutionResult(new BuildOperationParameters(targetGradleVersion, parameters.isEmbedded()), output.toString(), tasks);
+    }
+
+    private GradleVersion determineTargetGradleVersion(ProjectConnection connection) {
+        BuildEnvironment buildEnvironment = connection.getModel(BuildEnvironment.class);
+        return GradleVersion.version(buildEnvironment.getGradle().getGradleVersion());
     }
 
     private static OutputStream teeOutput(OutputStream capture, OutputStream user) {
